
#include <ESP8266WiFi.h>
#include <Wire.h>
#include "Gsender.h"

const char* ssid = "ssid of your local wifi network";
const char* password = "password of your local wifi network";
const char* to_email = "e-mail where you want receive notifications of movement detections";
const int addr_nano = 8;
const unsigned long interval = 1000;
unsigned long last_check;
unsigned long last_check_newclient;

// Create an instance of the server
// specify the port to listen on as an argument
WiFiServer server(80);

const int max_number_of_slave_devices = 4;

#include <Time.h>
#include <TimeLib.h>
#include "DS3231.h"

RTClib RTC;
WiFiClient client;

boolean const_network_presence_connected = true;
boolean const_network_movement_connected = true;
boolean const_presence_command = false;
boolean const_ir_command = false;
int device_to_test = -1;
String last_action = "N";
int counter_movements[max_number_of_slave_devices+1];
String last_movement[max_number_of_slave_devices+1];
String presencecommand = "";
String ircommand = "";

void printDate(time_t  t)
{
    Serial.print(year(t)) ;    Serial.print(+ "/") ;   Serial.print(month(t));   Serial.print(+ "/") ;
    Serial.print(day(t));    Serial.print( " ") ;
    Serial.print(hour(t));   Serial.print(+ ":") ;   Serial.print(minute(t));   Serial.print(":") ;
    Serial.println(second(t));
}

String strDate(time_t  t)
{
  String strD = "";

  strD = strD + String(year(t));
  if (String(month(t)).length() == 1) strD = strD + "-0" + String(month(t));
  else strD = strD + "-" + String(month(t));  
  if (String(day(t)).length() == 1) strD = strD + "-0" + String(day(t));
  else strD = strD + "-" + String(day(t));
  if (String(hour(t)).length() == 1) strD = strD + " 0" + String(hour(t));
  else strD = strD + " " + String(hour(t));
  if (String(minute(t)).length() == 1) strD = strD + ":0" + String(minute(t));
  else strD = strD + ":" + String(minute(t));
  if (String(second(t)).length() == 1) strD = strD + ":0" + String(second(t));
  else strD = strD + ":" + String(second(t));
  return strD;
  
}

void setup() {
  Serial.begin(9600);
  delay(10);

  last_check = millis();

  // Connect to WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");

  // Start the server
  server.begin();
  Serial.println("Server started");

  // Print the IP address
  Serial.println(WiFi.localIP());

  //Wire.begin(D1, D2);           /* join i2c bus with SDA=D1 and SCL=D2 of NodeMCU */
  Wire.begin();

  DateTime current_date = RTC.now();
  setTime(current_date.hour(),current_date.minute(),current_date.second(),
          current_date.day(),current_date.month(),current_date.year());  

  for (int i=0;i<=max_number_of_slave_devices;i++) counter_movements[i]=0;
  
}

boolean SendEmail(String subject, String message) {
  Gsender *gsender = Gsender::Instance();    // Getting pointer to class instance
  if(gsender->Subject(subject)->Send(to_email, message)) {
      Serial.println("Message send.");
      return true;
  } else {
      Serial.print("Error sending message: ");
      Serial.println(gsender->getError());
      return false;
  }
}

void printWebDateTime(String str_last_action) {
  client.print(str_last_action.substring(5,7));
  client.print("-");
  client.print(str_last_action.substring(3,5));
  client.print("-");
  client.print(str_last_action.substring(1,3));
  client.print(" ");
  client.print(str_last_action.substring(7,9));
  client.print(":");
  client.print(str_last_action.substring(9,11));
  client.print(":");
  client.print(str_last_action.substring(11,13));
}

void printWebDevice(String str_last_action) {
  client.print(" (DEVICE ");
  client.print(str_last_action.substring(0,1));
  client.print(")");
}

void loop() {

  String commStringS1 = "";   // MOVEMENT
  String commStringS2 = "";   // LASTACTION
  
  unsigned long time_now = millis();
  if (time_now-last_check >= interval) {  
  
    printDate (now());
    
    Wire.requestFrom(addr_nano, 30);    // request 30 bytes from slave device #8
    delay(30);
    String commStrintI2C = "";
    while (Wire.available()) { // slave may send less than requested
      char c = Wire.read(); // receive a byte as character
      //Serial.print(c);         // print the character
      commStrintI2C = commStrintI2C + c;
    }
    //Serial.println(" ");
    commStringS1 = commStrintI2C.substring(0,13);
    Serial.println(commStringS1);
    commStringS2 = commStrintI2C.substring(13,26);
    Serial.println(commStringS2);
    int address_slave;
    char addr[1];
    commStringS1.substring(0,1).toCharArray(addr,2); 
    if (isDigit(addr[0])) {
      address_slave = commStringS1.substring(0,1).toInt();
      counter_movements[address_slave]++;
      last_movement[address_slave] = commStringS1;
      SendEmail("¡Movement detected by device " + String(address_slave) + "!", "Movement detected by device " + String(address_slave) + " (" + strDate(now()) + ")");
    }
    commStringS2.substring(0,1).toCharArray(addr,2);
    if (isDigit(addr[0])) last_action = commStringS2;
    
    Wire.beginTransmission(addr_nano); // transmit to device #8
    String commI2C = "";
    char commEnd[35];
     
    if (const_network_presence_connected) commI2C = "1";
    else commI2C = "2";
    if (const_network_movement_connected) commI2C = commI2C + "1";
    else commI2C = commI2C + "2";
    if (device_to_test > -1) commI2C = commI2C + "1";
    else commI2C = commI2C + "2";
    if (device_to_test > -1) commI2C = commI2C + String(device_to_test);
    else commI2C = commI2C + "0";
    if (const_presence_command) commI2C = commI2C + 'P' + presencecommand;
    if (const_ir_command) commI2C = commI2C + 'I' + ircommand;
    
    commI2C.toCharArray(commEnd,36);
    Wire.write(commEnd);                 // sends five bytes
    Wire.endTransmission();           // stop transmitting
    
    last_check = time_now;
  }

  // Check if a client has connected
  client = server.available();
  if (!client) {
    return;
  }

  // Wait until the client sends some data
  Serial.println("new client");
  unsigned long last_check_newclient = millis();
  time_now = millis();
  while (!client.available() and (time_now-last_check_newclient < 3000)) {
    delay(100);
    time_now = millis();
  }
  if (time_now-last_check_newclient >= interval) return;

  String linea1 = client.readStringUntil('r');
  Serial.println(linea1);

  const_presence_command = false;
  const_ir_command = false;

  if (linea1.indexOf("PRESENCEON")>0) {
    const_network_presence_connected = true;
  }
  
  if (linea1.indexOf("PRESENCEOFF")>0) {
    const_network_presence_connected = false;
    last_action = "N";
  }
  
  if (linea1.indexOf("MOVEMENTON")>0) {
    const_network_movement_connected = true;
  }
  
  if (linea1.indexOf("MOVEMENTOFF")>0) {
    const_network_movement_connected = false;
    for (int i=0;i<=max_number_of_slave_devices;i++) counter_movements[i]=0;
  }

  if (linea1.indexOf("TESTOFF")>0) {
    device_to_test = -1;
    const_network_movement_connected = true;
    const_network_presence_connected = true;
  }
  
  for (int inode = 0; inode <= max_number_of_slave_devices;inode++)
    if (linea1.indexOf("TEST" + String(inode))>0) device_to_test = inode;
  if (device_to_test >= 0) {
    const_network_movement_connected = false;
    const_network_presence_connected = false;
  }
  
  if (linea1.indexOf("PRESENCECOMMAND")>0) {
    int dotPosicion = 0;
    String linea2 = linea1.substring(linea1.indexOf("/") + 16,linea1.indexOf("H") - 1);
    String subreceived = "";
    int number = 0;
    bool exit_while = false;

    subreceived = linea2.substring(dotPosicion,linea2.length());
    while ((subreceived.length() > 0) and (not (exit_while))) {
      Serial.print("***** SUBREC: ");
      Serial.println(subreceived);
      if (subreceived.indexOf(",") > 0)
        dotPosicion = subreceived.indexOf(",");
      else
        dotPosicion = subreceived.length();
      String checknumber = subreceived.substring(0,dotPosicion);
      Serial.print("***** NUMBER: ");
      Serial.println(checknumber);
      int iter = 0;
      while (not (exit_while) and (iter < checknumber.length())) {
        char c;
        c = checknumber[iter];
        if (not (isDigit(c))) exit_while = true;
        iter++;
      } 
      if (not (exit_while)) {
        number++;
        subreceived = subreceived.substring(dotPosicion+1,linea2.length());
      }
    }
    Serial.print("**** TOTNUMBER: ");
    Serial.println(number);
    if (number == 9) {
      presencecommand = linea2;
      const_presence_command = true;  
    }
    else {
      presencecommand = "";
      const_presence_command = false;
    }
  } else {
    const_presence_command = false;
  }

  if (linea1.indexOf("IRCOMMAND")>0) {
    int dotPosicion = 0;
    String linea2 = linea1.substring(linea1.indexOf("/") + 10,linea1.indexOf("H") - 1);
    String subreceived = "";
    int number = 0;
    bool exit_while = false;

    subreceived = linea2.substring(dotPosicion,linea2.length());
    while ((subreceived.length() > 0) and (not (exit_while))) {
      Serial.print("***** SUBREC: ");
      Serial.println(subreceived);
      if (subreceived.indexOf(",") > 0)
        dotPosicion = subreceived.indexOf(",");
      else
        dotPosicion = subreceived.length();
      String checknumber = subreceived.substring(0,dotPosicion);
      Serial.print("***** NUMBER: ");
      Serial.println(checknumber);
      if (not (number == 1)) {  // protocol ID
        int iter = 0;
        while (not (exit_while) and (iter < checknumber.length())) {
          char c;
          c = checknumber[iter];
          if ((number == 0) or (number == 2) or (number == 3)) if (not (isDigit(c))) exit_while = true;
          if (number == 4) if (not (isHexadecimalDigit(c))) exit_while = true;
          iter++;
        } 
        if (not (exit_while)) {
          number++;
          subreceived = subreceived.substring(dotPosicion+1,linea2.length());
        }
      } else {
        if ((checknumber == "SONY") or
            (checknumber == "NEC") or
            (checknumber == "RC5") or
            (checknumber == "RC6") or
            (checknumber == "LG") or
            (checknumber == "JVC") or
            (checknumber == "WHYNTER") or
            (checknumber == "SAMSUNG") or
            (checknumber == "DISH") or
            (checknumber == "DENON") or
            (checknumber == "SHARP") or
            (checknumber == "LEGO_PF")
            ) {
              subreceived = subreceived.substring(dotPosicion+1,linea2.length());
              number++; 
            } else exit_while = true;
      }
    }
    Serial.print("**** TOTNUMBER: ");
    Serial.println(number);
    linea2.toUpperCase();
    if (number == 5) {
      ircommand = linea2;
      const_ir_command = true;
    }
    else {
      ircommand = "";
      const_ir_command = false;
    }
  } else {
    const_ir_command = false;
  }
  
  client.flush();
  Serial.println("Enviando respuesta...");   
  client.println("HTTP/1.1 200 OK");
  client.println("Content-Type: text/html");
  client.println("Connection: close");// La conexión se cierra después de finalizar de la respuesta
  client.println();  
  client.println("<!DOCTYPE HTML>");
  client.println("<html>");

  //STATE SECTION
  client.println("<p style=""""text-align: center;""""><span style=""""background-color:Lime;""""><code><span style=""""font-size:60px;"""">SYSTEM STATE</span></code></span></p>");

  client.print("<p><center><bold><FONT FACE=""""impact"""" SIZE=7 COLOR=""""#8A2BE2"""">DATE TIME: ");
  client.print(strDate(now()));
  client.println("</FONT></bold></center></p>");
  
  if (device_to_test < 0) {
    if (const_network_presence_connected) {
      client.println("<p><span style=""""color:#008000"""";""""font-size:33px;""""><code><img alt=""""yes"""" height=""""60"""" src=""""http://bestonlinehtmleditor.com/ckeditor/plugins/smiley/images/thumbs_up.gif"""" title=""""yes"""" width=""""60"""" />PRESENCE SIMULATOR ENABLED</code></span></p>");
      if (not (last_action[0] == 'N')) {
        client.print("<center><bold><FONT FACE=""""impact"""" SIZE=4 COLOR=""""#8A2BE2"""">LAST: ");
        printWebDateTime(last_action);
        printWebDevice(last_action);
        client.println("</FONT></bold></center>");
      } else {
        client.print("<center><bold><FONT FACE=""""impact"""" SIZE=4 COLOR=""""#8A2BE2"""">LAST: waiting ...");
        client.println("</FONT></bold></center>");      
      }      
    }
    else {
      client.println("<p><span style=""""color:#FF0000"""";""""font-size:33px;""""><code><img alt=""""no"""" height=""""60"""" src=""""http://bestonlinehtmleditor.com/ckeditor/plugins/smiley/images/thumbs_down.gif"""" title=""""no"""" width=""""60"""" />PRESENCE SIMULATOR DISABLED</code></span></p>");
    }
    if (const_network_movement_connected) {
      client.println("<p><span style=""""color:#008000"""";""""font-size:33px;""""><code><img alt=""""yes"""" height=""""60"""" src=""""http://bestonlinehtmleditor.com/ckeditor/plugins/smiley/images/thumbs_up.gif"""" title=""""yes"""" width=""""60"""" />MOVEMENT DETECTOR ENABLED</code></span></p>");
      boolean exist_movement = false;
      for (int i=0;i<=max_number_of_slave_devices;i++) {
        if (counter_movements[i] > 0) {
          exist_movement = true;
          client.print("<p><bold><FONT FACE=""""impact"""" SIZE=4 COLOR=""""RED"""">");
          printWebDevice(last_movement[i]);
          client.print(": ");
          client.print(counter_movements[i]);
          client.print(" (");
          printWebDateTime(last_movement[i]);
          client.print(")");
          client.println("</FONT></bold></p>");            
        }
      }
      if (not exist_movement) {
        client.print("<center><bold><FONT FACE=""""impact"""" SIZE=4 COLOR=""""#8A2BE2"""">No movements detected");
        client.println("</FONT></bold></center>");              
      }
    }
    else {
      client.println("<p><span style=""""color:#FF0000"""";""""font-size:33px;""""><code><img alt=""""no"""" height=""""60"""" src=""""http://bestonlinehtmleditor.com/ckeditor/plugins/smiley/images/thumbs_down.gif"""" title=""""no"""" width=""""60"""" />MOVEMENT DETECTOR DISABLED</code></span></p>");
    }
    
      //COMMANDS SECTION
    client.println("<p style=""""text-align: center;""""><span style=""""background-color:Lime;""""><code><span style=""""font-size:60px;"""">COMMANDS</span></code></span></p>");
    client.println("<div style='text-align:center;'>");
    if (const_network_presence_connected) client.println("<p style=""""text-align: center;""""><input name=""""btn_valor1"""" style=""""font-size:60px;"""" type=""""button"""" onclick=""""location.href='PRESENCEOFFr';"""" value=""""DISABLED_PRESENCE"""" /></p>");
    else client.println("<p style=""""text-align: center;""""><input name=""""btn_valor2"""" style=""""font-size:60px;"""" type=""""button"""" onclick=""""location.href='PRESENCEONr';"""" value=""""ENABLED_PRESENCE"""" /></p>");
    if (const_network_movement_connected) client.println("<p style=""""text-align: center;""""><input name=""""btn_valor1"""" style=""""font-size:60px;"""" type=""""button"""" onclick=""""location.href='MOVEMENTOFFr';"""" value=""""DISABLED_MOVEMENT"""" /></p>");
    else client.println("<p style=""""text-align: center;""""><input name=""""btn_valor2"""" style=""""font-size:60px;"""" type=""""button"""" onclick=""""location.href='MOVEMENTONr';"""" value=""""ENABLED_MOVEMENT"""" /></p>");
    client.println("</div>");
    
  } else {
    client.print("<p><center><bold><FONT FACE=""""impact"""" SIZE=7 COLOR=""""#8A2BE2"""">Testing device ");
    client.print(String(device_to_test));
    client.println("</FONT></bold></center></p>");
  }

  client.println("<form name=""""link""""><center><select name=""""menu"""" style=""""font-size:60px"""">");
  client.println("<option selected value='TESTOFFr'> Test mode OFF </option>");
  for (int i=0;i<=max_number_of_slave_devices;i++) {
    client.print("<option value='TEST");
    client.print(String(i));
    client.print("r'> Test device ");
    client.print(String(i));
    if (i==0) client.print(" (master) ");
    client.print("</option>");
  }  
  client.print("</select>");
  client.println("<input style=""""font-size:60px"""" onclick=""""location.href=document.link.menu.options[document.link.menu.selectedIndex].value;"""" type=""""button"""" value="""" Send """" /></center></form>");

  client.println("<p style=""""text-align: center;""""><span style=""""background-color:Lime;""""><code><span style=""""font-size:60px;"""">PRESENCE COMMAND</span></code></span></p>");
  client.print("<form name=""""finput""""><center>");
  if (not(presencecommand == "")) {
    client.print("<p><center><bold><FONT FACE=""""impact"""" SIZE=6 COLOR=""""#8A2BE2"""">LAST: ");
    client.print(presencecommand);
    client.println("</FONT></bold></center></p>");  
  } else {
    client.print("<p><center><bold><FONT FACE=""""impact"""" SIZE=6 COLOR=""""#8A2BE2"""">LAST: NOT VALID");
    client.println("</FONT></bold></center></p>");    
  }  
  client.print("<input type=""""text"""" name=""""pinput"""" style=""""font-size:60px"""">");
  client.println("<input style=""""font-size:60px"""" onclick=""""location.href='PRESENCECOMMAND'+document.finput.pinput.value;"""" type=""""button"""" value="""" Send """" /></center></form>");
  
  client.println("<p style=""""text-align: center;""""><span style=""""background-color:Lime;""""><code><span style=""""font-size:60px;"""">IR CODE COMMAND</span></code></span></p>");
  client.print("<form name=""""finput2""""><center>");
  if (not(ircommand == "")) {
    client.print("<p><center><bold><FONT FACE=""""impact"""" SIZE=6 COLOR=""""#8A2BE2"""">LAST: ");
    client.print(ircommand);
    client.println("</FONT></bold></center></p>");  
  } else {
    client.print("<p><center><bold><FONT FACE=""""impact"""" SIZE=6 COLOR=""""#8A2BE2"""">LAST: NOT VALID");
    client.println("</FONT></bold></center></p>");    
  }    
  client.print("<input type=""""text"""" name=""""pinput2"""" style=""""font-size:60px"""">");
  client.println("<input style=""""font-size:60px"""" onclick=""""location.href='IRCOMMAND'+document.finput2.pinput2.value;"""" type=""""button"""" value="""" Send """" /></center></form>");
  

  delay(1);
  Serial.println("Client disconnected");
    
}
